<?php

namespace Cobblestone\CcLogin\Service;

use Exception;

class UserService extends AbstractCCService
{
    /**
     * @var string
     */
    protected $apiUrl;


    /**
     * UserService constructor.
     * @param string $apiUrl
     */
    public function __construct(string $apiUrl)
    {
        $this->apiUrl = $apiUrl;
    }

    /**
     * @param string $token
     * @return array
     * @throws Exception
     */
    public function getProfileData(string $token)
    {
        return $this->callURMService("/me", $token);
    }

}